import aiohttp
from .const import BASE_URL, INFO_STATION_URL
from .exceptions import BadRequestError, ForbiddenError, TooManyRequestsError, InternalServerError, UnknownAPIError

class MeteocatInfoStation:
    """Clase para interactuar con la lista de municipios de la API de Meteocat."""

    def __init__(self, api_key: str):
        """
        Inicializa la clase MeteocatTown.

        Args:
            api_key (str): Clave de API para autenticar las solicitudes.
        """
        self.api_key = api_key
        self.headers = {
            "Content-Type": "application/json",
            "X-Api-Key": self.api_key,
        }

    async def get_infostation(self, station_id: str):
        """
        Obtiene los metadatos de la estación desde la API de Meteocat.

        Returns:
            dict: Datos de la estación. Nombre, coordenadas, altitud, provincia, comarca, etc.
        """
        url = f"{BASE_URL}{INFO_STATION_URL}".format(
            codi_estacio=station_id
        )
        async with aiohttp.ClientSession() as session:
            try:
                async with session.get(url, headers=self.headers) as response:
                    if response.status == 200:
                        return await response.json()

                    # Gestionar errores según el código de estado
                    if response.status == 400:
                        raise BadRequestError(await response.json())
                    elif response.status == 403:
                        error_data = await response.json()
                        if error_data.get("message") == "Forbidden":
                            raise ForbiddenError(error_data)
                        elif error_data.get("message") == "Missing Authentication Token":
                            raise ForbiddenError(error_data)
                    elif response.status == 429:
                        raise TooManyRequestsError(await response.json())
                    elif response.status == 500:
                        raise InternalServerError(await response.json())
                    else:
                        raise UnknownAPIError(f"Unexpected error {response.status}: {await response.text()}")
            
            except aiohttp.ClientError as e:
                raise UnknownAPIError(
                    message=f"Error al conectar con la API de Meteocat: {str(e)}",
                    status_code=0,
                )

            except Exception as ex:
                raise UnknownAPIError(
                    message=f"Error inesperado: {str(ex)}",
                    status_code=0,
                )



import aiohttp
import logging
from .const import (
    BASE_URL,
    UVI_DATA_URL
)
from .exceptions import (
    BadRequestError,
    ForbiddenError,
    TooManyRequestsError,
    InternalServerError,
    UnknownAPIError,
)

_LOGGER = logging.getLogger(__name__)

class MeteocatUviData:
    """Clase para interactuar con los datos del índice UVI de la API de Meteocat."""

    def __init__(self, api_key: str):
        """
        Inicializa la clase MeteocatUviData.

        Args:
            api_key (str): Clave de API para autenticar las solicitudes.
        """
        self.api_key = api_key
        self.headers = {
            "Content-Type": "application/json",
            "X-Api-Key": self.api_key,
        }

    async def get_uvi_index(self, town_id: str):
        """
        Obtiene los datos del índice UVI para un municipio.

        Args:
            town_id (str): Código del municipio.

        Returns:
            dict: Datos del índice UVI del municipio.
        """
        if not town_id:
            raise ValueError("El parámetro 'town_id' no puede estar vacío.")

        url = f"{BASE_URL}{UVI_DATA_URL}".format(
            codi_municipi=town_id
        )
    
        async with aiohttp.ClientSession() as session:
            try:
                async with session.get(url, headers=self.headers) as response:
                    if response.status == 200:
                        return await response.json()

                    # Gestionar errores según el código de estado
                    if response.status == 400:
                        raise BadRequestError(await response.json())
                    elif response.status == 403:
                        error_data = await response.json()
                        if error_data.get("message") == "Forbidden":
                            raise ForbiddenError(error_data)
                        elif error_data.get("message") == "Missing Authentication Token":
                            raise ForbiddenError(error_data)
                    elif response.status == 429:
                        raise TooManyRequestsError(await response.json())
                    elif response.status == 500:
                        raise InternalServerError(await response.json())
                    else:
                        raise UnknownAPIError(
                            f"Unexpected error {response.status}: {await response.text()}"
                        )

            except aiohttp.ClientError as e:
                raise UnknownAPIError(
                    message=f"Error al conectar con la API de Meteocat: {str(e)}",
                    status_code=0,
                )

            except Exception as ex:
                raise UnknownAPIError(
                    message=f"Error inesperado: {str(ex)}",
                    status_code=0,
                )



"""meteocatpy constants."""
BASE_URL = "https://api.meteo.cat"
MUNICIPIS_LIST_URL = "/referencia/v1/municipis"
MUNICIPIS_HORA_URL = "/pronostic/v1/municipalHoraria/{codi}"
MUNICIPIS_DIA_URL = "/pronostic/v1/municipal/{codi}"
SYMBOLS_URL = "/referencia/v1/simbols"
QUOTES_URL = "/quotes/v1/consum-actual"
STATIONS_LIST_URL = "/xema/v1/estacions/metadades"
INFO_STATION_URL = "/xema/v1/estacions/{codi_estacio}/metadades"
STATIONS_MUNICIPI_URL = "/xema/v1/representatives/metadades/municipis/{codi_municipi}/variables/{codi_variable}"
VARIABLES_URL = "/xema/v1/variables/mesurades/metadades"
STATION_DATA_URL = "/xema/v1/estacions/mesurades/{codiEstacio}/{any}/{mes}/{dia}"
UVI_DATA_URL = "/pronostic/v1/uvi/{codi_municipi}"
ALERTS_URL = "/pronostic/v2/smp/episodis-oberts?data={any}-{mes}-{dia}Z"
LIGHTNING_URL = "/xdde/v1/informes/comarques/{codi_comarca}/{any}/{mes}/{dia}"


import aiohttp
from .const import BASE_URL, SYMBOLS_URL
from .exceptions import BadRequestError, ForbiddenError, TooManyRequestsError, InternalServerError, UnknownAPIError


class MeteocatSymbols:
    """Clase para interactuar con la API de símbolos de Meteocat."""

    def __init__(self, api_key: str):
        """
        Inicializa la clase MeteocatSymbols.

        Args:
            api_key (str): Clave de API para autenticar las solicitudes.
        """
        self.api_key = api_key
        self.headers = {
            "Content-Type": "application/json",
            "X-Api-Key": self.api_key,
        }
        self.symbols_map = {}

    async def fetch_symbols(self):
        url = f"{BASE_URL}{SYMBOLS_URL}"
        async with aiohttp.ClientSession() as session:
            try:
                async with session.get(url, headers=self.headers) as response:
                    if response.status == 200:
                        data = await response.json()
                        print(data)  # Esto te mostrará la estructura completa en la consola

                        # Asegurarse de que `data` sea una lista de categorías
                        if isinstance(data, list):
                            self.symbols_map = {}
                            for category in data:
                                if "valors" in category:
                                    # Guardamos los valores de cada categoría
                                    self.symbols_map[category["nom"]] = category["valors"]
                            return data  # Devolvemos todo el conjunto de datos

                        else:
                            raise UnknownAPIError(f"Unexpected structure of data: {data}", status_code=response.status)

                    # Gestionar errores de respuesta
                    if response.status == 400:
                        raise BadRequestError(await response.json())
                    elif response.status == 403:
                        error_data = await response.json()
                        if error_data.get("message") == "Forbidden":
                            raise ForbiddenError(error_data)
                        elif error_data.get("message") == "Missing Authentication Token":
                            raise ForbiddenError(error_data)
                    elif response.status == 429:
                        raise TooManyRequestsError(await response.json())
                    elif response.status == 500:
                        raise InternalServerError(await response.json())
                    else:
                        raise UnknownAPIError(f"Unexpected error {response.status}: {await response.text()}", status_code=response.status)

            except aiohttp.ClientError as e:
                raise UnknownAPIError(
                    message=f"Error al conectar con la API de Meteocat: {str(e)}",
                    status_code=0,
                )

            except Exception as ex:
                raise UnknownAPIError(
                    message=f"Error inesperado: {str(ex)}",
                    status_code=0,
                )



    def get_description(self, category: str, code: int) -> str:
        """
        Obtiene la descripción de un código de símbolo dentro de una categoría.

        Args:
            category (str): Nombre de la categoría (e.g., "cel").
            code (int): Código del símbolo.

        Returns:
            str: Descripción del símbolo. Retorna 'Desconocido' si el código no está en el mapeo.
        """
        category_symbols = self.symbols_map.get(category, [])
        for symbol in category_symbols:
            if symbol["codi"] == str(code):  # El código es devuelto como string por la API
                return symbol["descripcio"]
        return "Desconocido"



import aiohttp
from .stations import MeteocatStations  # Importamos la clase MeteocatStations
from .const import BASE_URL, STATIONS_MUNICIPI_URL
from .exceptions import (
    BadRequestError,
    ForbiddenError,
    TooManyRequestsError,
    InternalServerError,
    UnknownAPIError,
)

class MeteocatTownStations:
    """
    Clase para interactuar con la API de Meteocat y obtener
    las estaciones representativas de un municipio para una variable específica.
    """

    def __init__(self, api_key: str):
        """
        Inicializa la clase MeteocatTownStations.

        Args:
            api_key (str): Clave de API para autenticar las solicitudes.
        """
        self.api_key = api_key
        self.headers = {
            "Content-Type": "application/json",
            "X-Api-Key": self.api_key,
        }
        self.stations_service = MeteocatStations(api_key)  # Instancia de MeteocatStations

    async def get_town_stations(self, town_id: str, variable_id: str):
        """
        Obtiene la lista de estaciones representativas para un municipio y una variable específica,
        enriqueciendo los datos con el nombre de las estaciones.

        Args:
            codi_municipi (str): Código del municipio.
            codi_variable (str): Código de la variable.

        Returns:
            list: Datos de las estaciones representativas con nombres añadidos.
        """
        # Obtener la lista completa de estaciones
        all_stations = await self.stations_service.get_stations()

        # Crear un diccionario para acceder rápidamente a los nombres por código
        station_names = {station["codi"]: station["nom"] for station in all_stations}

        # URL para obtener las estaciones del municipio y la variable
        url = f"{BASE_URL}{STATIONS_MUNICIPI_URL}".format(
            codi_municipi=town_id, codi_variable=variable_id
        )

        async with aiohttp.ClientSession() as session:
            try:
                async with session.get(url, headers=self.headers) as response:
                    if response.status == 200:
                        data = await response.json()

                        # Enriquecer el JSON con los nombres de las estaciones
                        for town in data:
                            for variable in town.get("variables", []):
                                for station in variable.get("estacions", []):
                                    codi = station["codi"]
                                    station["nom"] = station_names.get(codi, "Nombre desconocido")

                        return data

                    # Gestionar errores según el código de estado
                    if response.status == 400:
                        raise BadRequestError(await response.json())
                    elif response.status == 403:
                        error_data = await response.json()
                        if error_data.get("message") == "Forbidden":
                            raise ForbiddenError(error_data)
                        elif error_data.get("message") == "Missing Authentication Token":
                            raise ForbiddenError(error_data)
                    elif response.status == 429:
                        raise TooManyRequestsError(await response.json())
                    elif response.status == 500:
                        raise InternalServerError(await response.json())
                    else:
                        raise UnknownAPIError(
                            f"Unexpected error {response.status}: {await response.text()}"
                        )

            except aiohttp.ClientError as e:
                raise UnknownAPIError(
                    message=f"Error al conectar con la API de Meteocat: {str(e)}",
                    status_code=0,
                )

            except Exception as ex:
                raise UnknownAPIError(
                    message=f"Error inesperado: {str(ex)}",
                    status_code=0,
                )




"""Meteocat Helpers."""

from datetime import datetime
from typing import Any
import re
import unicodedata
from zoneinfo import ZoneInfo

TZ_UTC = ZoneInfo("UTC")


def dict_nested_value(data: dict[str, Any] | None, keys: list[str] | None) -> Any:
    """Get value from dict with nested keys."""
    if keys is None or len(keys) == 0:
        return None
    for key in keys or {}:
        if data is not None:
            data = data.get(key)
    return data


def get_current_datetime(tz: ZoneInfo = TZ_UTC, replace: bool = True) -> datetime:
    """Return current datetime in UTC."""
    cur_dt = datetime.now(tz=tz)
    if replace:
        cur_dt = cur_dt.replace(minute=0, second=0, microsecond=0)
    return cur_dt


def parse_api_timestamp(timestamp: str, tz: ZoneInfo = TZ_UTC) -> datetime:
    """Parse API timestamp into datetime."""
    return datetime.fromisoformat(timestamp).replace(tzinfo=tz)


def slugify(value: str, allow_unicode: bool = False) -> str:
    """Convert string to a valid file name."""
    if allow_unicode:
        value = unicodedata.normalize("NFKC", value)
    else:
        value = (
            unicodedata.normalize("NFKD", value)
            .encode("ascii", "ignore")
            .decode("ascii")
        )
    value = re.sub(r"[^\w\s]", "-", value.lower())
    return re.sub(r"[-\s]+", "-", value).strip("-_")
